from setuptools import setup

setup(
    name='aiogear',
    version='0.2.1',
    author='Sinan Nalkaya',
    author_email='sardok@gmail.com',
    url='https://github.com/sardok/aiogear',
    description='Asynchronous gearman protocol based on asyncio',
    packages=['aiogear'],
    classifiers=[
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ]
)
